<?php
namespace MyMenu\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Session\Session;
use MyMenu\Libraries\KeyValueStorage;

class Account
{

    private $crudService;
    private $sessionManager;
    private $environment;
    private $connection;

    public function __construct($crudService, Session $sessionManager, $environment, $entityManager)
    {
        $this->crudService = $crudService;
        $this->sessionManager = $sessionManager;
        $this->environment = $environment;
        $this->connection = $entityManager->getConnection();
    }

    public function createAccount($request, $response)
    {
        $accountData = $request->getRow();

        if ((isset($accountData['phoneNum']) && strlen($accountData['phoneNum']) > 24) || !isset($accountData['phoneNum'])) {
            $accountData['phoneNum'] = '';
        }

        $row = array(
            '__is_new' => true,
            '__new_code_pk' => true,
            '__cast_number' => true,
            '__new_code_pk' => 'USUARIOAPP',
            '__table_pk' => 'NRUSUARIOAPP',
            'DSEMAILUSUARIOAPP' => $accountData['email'],
            'DSSENHAUSUARIOAPP' => md5($accountData['password']),
            'DSTELEFONE' => $accountData['phoneNum'],
            'DSPRIMEIRONOME' => $accountData['name'],
            'DSULTIMONOME' => $accountData['lastname'],
            'FACEBOOK_ID' => null,
            'FACEBOOK_TOKEN' => null,
            'FACEBOOK_TOKEN_EXP_DATE' => null,
            'FOTO' => null,
            'CDVERIFICASENHA' => null,
        );

        $dataSetToSave = new DataSet('usuarioapp', array($row));
        $saveRequest = new DTO\Request\DataSet($dataSetToSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $rowsToSave = $this->crudService->getRowsToSave($saveRequest, 'usuarioapp');
        try {
            $this->crudService->save('usuarioapp', $rowsToSave);
        } catch (\Exception $e) {
            $uniqueErr = 'ORA-00001';
            $errorMsg = $e->getMessage();
            if (substr_count($errorMsg, $uniqueErr)) {
                $errorMsg = 'Email já cadastrado.';
            }
            $response->setError(new Error($errorMsg, 500));
        }
    }
    
    public function removeAccount($request, $response){
        $isAnon = KeyValueStorage::get('isAnon');
        if(!$isAnon){
            $userID = KeyValueStorage::get('NRUSUARIOAPP');
            $updateConsumo = "UPDATE AVALIACAOPRATO SET NRUSUARIOAPP = NULL, IDCONSUMO = 'N', NMUSUARIO = NULL, HRCONSUMO = NULL, DSCONSUMO = NULL WHERE NRUSUARIOAPP = :NRUSUARIOAPP AND DTCARDAPIO >= TRUNC(SYSDATE)";
            $statement = $this->connection->prepare($updateConsumo);
            $statement->execute(["NRUSUARIOAPP" => $userID]);
            
            $updateUserApp = "UPDATE USUARIOAPP SET DSEMAILUSUARIOAPP = 'USUARIO_REMOVIDO ' || systimestamp, DSSENHAUSUARIOAPP = NULL, FACEBOOK_ID = NULL, FACEBOOK_TOKEN = NULL, FACEBOOK_TOKEN_EXP_DATE = NULL, DSTELEFONE = NULL WHERE NRUSUARIOAPP = :NRUSUARIOAPP";
            $statement = $this->connection->prepare($updateUserApp);
            $statement->execute(["NRUSUARIOAPP" => $userID]);
            
            $updateAppUserApp = "UPDATE APP_USUARIOAPLICATIVO SET IDATIVO = 'N' WHERE NRUSUARIOAPP = :NRUSUARIOAPP AND NRAPLICATIVO = 1";
            $statement = $this->connection->prepare($updateAppUserApp);
            $statement->execute(["NRUSUARIOAPP" => $userID]);
            // $this->anonymizeUserRatings($userID);
            // $this->anonymizeReservations($userID);
            // $this->anonymizeSurveyData($userID);
            // $this->deleteCustomerServiceMessages($userID);
            // $this->deleteAppData($userID);
            // $deleteAccountSQL = "DELETE FROM USUARIOAPP WHERE NRUSUARIOAPP = :NRUSUARIOAPP";
        } else {
            throw new \Exception("Anon account cannot be removed");
        }
    }
    
    protected function getAnonUserID(){
        $findAnonUserIDSQL = "SELECT NRUSUARIOAPP FROM USUARIOAPP WHERE DSEMAILUSUARIOAPP = 'anonimo@anonimo.com'";
        $statement = $this->connection->prepare($findAnonUserIDSQL);
        $statement->execute();
        $result = $statement->fetchAll();
        return $result[0]["NRUSUARIOAPP"];
    }
    
    protected function anonymizeReservations($userID){
        $anonUserID = $this->getAnonUserID();
        $params = [
            "NRUSUARIOAPP" => $userID,
            "IDUSUARIOANONIMO" => $anonUserID
        ];
        $deleteReviewsSQL = "UPDATE AVALIACAOPRATO SET NRUSUARIOAPP = :IDUSUARIOANONIMO, NMUSUARIO = 'CONTA REMOVIDA', DSCONSUMO = null WHERE IDCONSUMO = 'S' AND NRUSUARIOAPP = :NRUSUARIOAPP";
        $statement = $this->connection->prepare($deleteReviewsSQL);
        $statement->execute($params);
        
        $deleteReviewsSQL = "UPDATE AVALIACAOSERVICO SET NRUSUARIOAPP = :IDUSUARIOANONIMO, NMUSUARIO = 'CONTA REMOVIDA', DSCONSUMO = null WHERE IDCONSUMO = 'S' AND NRUSUARIOAPP = :NRUSUARIOAPP";
        $statement = $this->connection->prepare($deleteReviewsSQL);
        $statement->execute($params);
    }
    
    protected function anonymizeUserRatings($userID){
        $anonUserID = $this->getAnonUserID();
        $params = [
            "NRUSUARIOAPP" => $userID,
            "IDUSUARIOANONIMO" => $anonUserID
        ];
        $deleteReviewsSQL = "UPDATE AVALIACAOPRATO SET NRUSUARIOAPP = :IDUSUARIOANONIMO, DSAVALIACAOPRATO = null WHERE IDCONSUMO = 'N' AND NRUSUARIOAPP = :NRUSUARIOAPP";
        $statement = $this->connection->prepare($deleteReviewsSQL);
        $statement->execute($params);
    }
    
    protected function deleteCustomerServiceMessages($userID){
        $params = ["NRUSUARIOAPP" => $userID];
        $deleteCustomerServiceMessagesSQL = "DELETE FROM FALECONOSCOAPP WHERE NRFALECONOSCOAPP IN (SELECT NRFALECONOSCOAPP FROM FALECONOSCOAPP F JOIN USUARIOAPP U ON U.DSEMAILUSUARIOAPP = F.DSEMAILCLIENTE WHERE U.NRUSUARIOAPP = :NRUSUARIOAPP)";
        $statement = $this->connection->prepare($deleteCustomerServiceMessagesSQL);
        $statement->execute($params);
    }
    
    protected function anonymizeSurveyData($userID){
        $anonUserID = $this->getAnonUserID();
        $params = [
            "NRUSUARIOAPP" => $userID,
            "IDUSUARIOANONIMO" => $anonUserID
        ];
        
        $deleteAnsweredSurveysSQL = "UPDATE EXECUCAOPESQ SET NRUSUARIOPESQ = :IDUSUARIOANONIMO WHERE NRUSUARIOPESQ = :NRUSUARIOAPP";
        $statement = $this->connection->prepare($deleteAnsweredSurveysSQL);
        $statement->execute($params);
    }
    
    protected function deleteAppData($userID){
        $params = ["NRUSUARIOAPP" => $userID];
        $deleteAppDataSQL = "DELETE FROM APP_USUARIOAPLICATIVO WHERE NRUSUARIOAPP = :NRUSUARIOAPP";
        $statement = $this->connection->prepare($deleteAppDataSQL);
        $statement->execute($params);
    }
    
}
