function PesquisaController(ScreenService, General, templateManager, metaDataFactory, MessageService) {
    var pesquisaContainerName;
    var pesquisaContainer;
    var pesquisasWidget;
    var currentPesq;

    this.openItemPesquisa = function () {
        ScreenService.openWindow("product#item_pesquisa");
    };
    this.closeItemPesquisa = function () {
        ScreenService.openWindow("product#pesquisa");
    };

    this.itemPesquisaOnOpen = function (itemPesquisaWidget) {
        var containers = templateManager.containers;
        var historyItems = templateManager.historyItems;


        if (containers && historyItems)
            pesquisaContainerName = historyItems[historyItems.length - 2];
        pesquisaContainer = containers[pesquisaContainerName];

        if (pesquisaContainer) {
            pesquisasWidget = pesquisaContainer.getActiveWidget();
            currentPesq = pesquisasWidget.currentRow;
            itemPesquisaWidget.container.label = currentPesq.NMQUESTIONARIO;
            itemPesquisaWidget.label = currentPesq.NMQUESTIONARIO;

            clearFields(itemPesquisaWidget);

            var filter = [{
                "name": "NRQUESTIONARIO",
                "operator": "=",
                "value": currentPesq.NRQUESTIONARIO
            }];
            itemPesquisaWidget.reload(filter).then(function () {
                buildFields(itemPesquisaWidget);
                itemPesquisaWidget.newRow();
                itemPesquisaWidget.currentRow.NRPROGRAMACAOPESQ = currentPesq.NRPROGRAMACAOPESQ;
                itemPesquisaWidget.currentRow.NRQUESTIONARIO = currentPesq.NRQUESTIONARIO;
            });

        }
    };

    function clearFields(itemPesquisaWidget) {
        itemPesquisaWidget.fields = [];
    }

    function buildFields(itemPesquisaWidget) {
        var templates = {
            ABERTA: 'field/textarea.html',
            FECHADA_UNI: 'field/select-native.html',
            FECHADA_MUL: 'mymenu#fields/select-autocomplete-multiple.html'
        };

        var layout ={
            EMOJI: 'mymenu#fields/layout/emoji.html',
            STAR: 'mymenu#fields/layout/star.html',
            RANGE: 'mymenu#fields/layout/range.html',
        };

        var fieldNames = [];
        itemPesquisaWidget.dataSource.data.forEach(function (itemQuestionario) {
            var fieldName = 'QUESTAO' + itemQuestionario.NRQUESTAO;

            if (fieldNames.indexOf(fieldName) === -1) {
                let canHaveNextQuestion = itemQuestionario.IDTIPOQUESTAO !== 'FECHADA_MUL';
                fieldNames.push(
                    buildField(
                        itemPesquisaWidget, 
                        itemQuestionario, 
                        getIDQuestao(itemQuestionario, templates, layout), 
                        true, 
                        undefined, 
                        canHaveNextQuestion
                ));
            } else {
                var field = itemPesquisaWidget.getField(fieldName);
                field.dataSource.data.push(buildResp(itemQuestionario));
            }
        });

        buildSubmmitBtn(itemPesquisaWidget);
    }

    function getIDQuestao(itemQuestionario, templates, layout) {
        if(itemQuestionario.IDTIPOQUESTAO == 'ABERTA' && itemQuestionario.IDLAYOUTRESP !== 'TEXT') {
            return layout[itemQuestionario.IDLAYOUTRESP];
        }

        return templates[itemQuestionario.IDTIPOQUESTAO];
    }

    /**
     * @param {*} itemPesquisaWidget 
     * @param {*} itemQuestionario 
     * @param {*} template 
     * @param {boolean} [shouldAppendToWidget] 
     * @param {string} newFieldName 
     * @param {boolean} [canHaveNextQuestion]
     * @returns {Field|string}
     */
    function buildField(itemPesquisaWidget, 
                            itemQuestionario, 
                            template, 
                            shouldAppendToWidget = true, 
                            newFieldName = undefined, 
                            canHaveNextQuestion = true) {
        var fieldName = newFieldName;
        if (!fieldName) {
            fieldName = 'QUESTAO' + itemQuestionario.NRQUESTAO;
        }
        
        var field = {
            name: fieldName,
            label: itemQuestionario.TXQUESTAO,
            isVisible: true,
            template: template,
            class: 12,
            fields: [{
                    name: "value",
                    isVisible: false,
                    showOnList: false
                },
                {
                    name: "nextQuestionID",
                    isVisible: false,
                    showOnList: false
                },
                {
                    name: "desc",
                    // label: "Opções",
                    isVisible: true,
                    showOnList: true
                }
            ],
            valueField: "value",
            descriptionField: "desc",
            // outData: {},
            preventAutoFocus: true,
            dataSource: {
                data: [
                    buildResp(itemQuestionario)
                ]
            },
            descriptionFields: ["desc"],
        };

        if(itemQuestionario.IDQUESTAOOBRIGA === 'S'){
            field.validations = {
                required:  true
            };
            field.hideEmptyValue = true;
        }

        if(canHaveNextQuestion){
            field.events = [{
                "name": "FieldOnChange",
                "code": "PesquisaController.setDescriptionOnCurrentRow(args);PesquisaController.loadDependentQuestion(args)"
            }];
        } else {
            field.events = [{
                "name": "FieldOnChange",
                "code": "PesquisaController.setDescriptionOnCurrentRow(args);"
            }];
        }

        if(typeof template === 'string' && !template.includes('textarea')){
            field.outData = {};
            field.outData["description|" + fieldName] = 'desc';
        }
        // field.outData[outDataPropertyName] = "NRPROXIMAQUESTAO";
        // field.outData.NRPROXIMAQUESTAO = outDataPropertyName;
        let factoredField = metaDataFactory.fieldFactory(field, itemPesquisaWidget, shouldAppendToWidget);
        if (shouldAppendToWidget) {
            return fieldName;
        } else {
            return factoredField;
        }
    }

    let setDescriptionSelectNative = (field) => {
        if(field.value()){
            let answerDescription = field.modelValue();
            field.widget.currentRow['description|' + field.name] = answerDescription;
        } else {
            delete field.widget.currentRow['description|' + field.name];
        }
    };

    // all description setters must be arrow functions in order for the scope to work correctly
    let descriptionSetters = {
        "field/select-native.html": setDescriptionSelectNative,
    };

    this.setDescriptionOnCurrentRow = (args) => {
        let field = args.owner.field;
        if(descriptionSetters.hasOwnProperty(field.template)){
            descriptionSetters[field.template].call(null, field);
        }
    };

    

    this.buildDependent = (dataSet, parentFieldName, widget) => {
        let pesquisasWidget = widget;
        let itemQuestionario = dataSet.dataset.qst_especifica;

        var templates = {
            ABERTA: 'field/textarea.html',
            ABERTA_TEXTO: 'field/textarea.html',
            FECHADA: 'field/select-native.html',
            FECHADA_UNI: 'field/select-native.html',
            FECHADA_MUL: 'mymenu#fields/select-autocomplete-multiple.html'
        };

        const IDTIPOQUESTAO = itemQuestionario.IDTIPOQUESTAO;
        const IDOPCAORESPOSTA = itemQuestionario.IDOPCAORESPOSTA;
        let questionType = IDTIPOQUESTAO;
        questionType = (IDTIPOQUESTAO === "FECHADA" && IDOPCAORESPOSTA === "MULTIPLAS" && "FECHADA_MUL") || questionType; 

        let canHaveNextQuestion = questionType !== 'FECHADA_MUL';
        let newField = buildField(pesquisasWidget, itemQuestionario, templates[questionType],
            false, 'QUESTAO' + itemQuestionario.NRQUESTAO + "_" + parentFieldName, canHaveNextQuestion);
        let parentFieldIndex = pesquisasWidget.fields.findIndex(field => field.name === parentFieldName);
        widget.fields.splice(parentFieldIndex + 1, 0, newField);
        newField.parentFieldName = parentFieldName;


        newField.dataSource.data = dataSet.dataset.resp_especifica.map(itemQuestionario => buildResp(itemQuestionario));
        // dataSet.dataset.resp_especifica.forEach(itemQuestionario => {
        //     // var fieldName = 'QUESTAO'+itemQuestionario.NRQUESTAO;    
        //     newField.dataSource.data.push(buildResp(itemQuestionario));
        // });

        // buildField(pesquisasWidget, );
    };

    this.removeDependentQuestion = (questionID, parentName, widget) => {
        // widget.fields.()
        let dependentField = widget.fields.find(field => field.name === ('QUESTAO' + questionID + '_' + parentName) &&
            field.parentFieldName === parentName);
        if (dependentField && dependentField.dependentID) {
            this.removeDependentQuestion(dependentField.dependentID, dependentField.name, widget);
            delete dependentField.dependentID;
            delete widget.currentRow[('QUESTAO' + questionID + '_' + parentName)];
        }
        widget.fields = widget.fields.filter(field => field !== dependentField); //.splice(, 1);
        delete widget.currentRow[dependentField.name];
    };

    let isEmpty = (valueToCheck) => valueToCheck === "null" || valueToCheck === null ||
        valueToCheck === 'undefined' || valueToCheck === undefined;

    this.loadDependentQuestion = function (args) {
        let fieldName = args.owner.field.name;
        let nextQuestionID;
        
        if (args.row[fieldName]) {
            let splited = args.row[fieldName].split("@@");
            nextQuestionID = splited[1];
        } else {
            nextQuestionID = null;
        }
        // args.row[fieldName] = splited[0];
        if (!isEmpty(nextQuestionID) && !args.owner.field.dependentID) {
            args.owner.field.dependentID = nextQuestionID;
            General.callRestEngine("/getNextQuestion", "Row", {
                qstid: nextQuestionID
            }).then((response) => this.buildDependent(response, fieldName, args.owner.field.widget));
        } else if (args.owner.field.dependentID && nextQuestionID !== args.owner.field.dependentID) {
            this.removeDependentQuestion(args.owner.field.dependentID, args.owner.field.name, args.owner.field.widget);
            delete args.owner.field.dependentID;
            if (!isEmpty(nextQuestionID)) {
                args.owner.field.dependentID = nextQuestionID;
                General.callRestEngine("/getNextQuestion", "Row", {
                    qstid: nextQuestionID
                }).then((response) => this.buildDependent(response, fieldName, args.owner.field.widget));
            }
        }
        
    };

    function buildResp(itemQuestionario) {
        return {
            value: itemQuestionario.NRRESPOSTA + "@@" + itemQuestionario.NRPROXIMAQUESTAO,
            desc: itemQuestionario.DSRESPOSTA,
        };
    }

    function buildSubmmitBtn(itemPesquisaWidget) {
        var field = {
            name: "submmit",
            label: "Enviar",
            isVisible: true,
            template: "field/button.html",
            class: 12,
            events: [{
                name: "FieldOnClick",
                code: "PesquisaController.pesquisaSubmmit(args.owner.owner.widget);"
            }]
        };
        metaDataFactory.fieldFactory(field, itemPesquisaWidget, true);
    }

    function stripATchar(row) {
        let stripedRow = {};
        for (const key in row) {
            if (row[key] && typeof row[key] === 'string') {
                stripedRow[key] = row[key].split("@@")[0];
            } else if(row[key] && Array.isArray(row[key])){
                stripedRow[key] = row[key].map(answer => answer.split("@@")[0]);    
            }
        }
        return stripedRow;
    }

    let textareaRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName]){
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = {
                value: null,
                description: currentRow[fieldName],
                parentAnswer: parentAnswer || null
            };
        }
    };
    
    let selectNativeRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName]){
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = {
                value: currentRow[fieldName].split("@@")[0],
                description: currentRow["description|" + fieldName],
                parentAnswer: parentAnswer || null
            };
        }
    };

    let selectAutocompleteMultipleRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName] && currentRow[fieldName].length > 0){
            const descriptionKey = 'description|' + fieldName;
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = currentRow[fieldName].map((selectedElement) => {
                return {
                    value: selectedElement.value.split("@@")[0],
                    description: selectedElement[descriptionKey],
                    parentAnswer: parentAnswer || null
                };
            });
        }
    };
    
    let layoutBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName]) {
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = {
                value: null,
                description: currentRow[fieldName],
                parentAnswer: parentAnswer || null
            };
        }
    };
    
    /*let rangeBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        
    };*/

    let rowBuilders = {
        "field/textarea.html": textareaRowBuilder,
        "field/select-native.html" : selectNativeRowBuilder, 
        "mymenu#fields/select-autocomplete-multiple.html" : selectAutocompleteMultipleRowBuilder,
        'mymenu#fields/layout/emoji.html': layoutBuilder,
        'mymenu#fields/layout/star.html': layoutBuilder,
        'mymenu#fields/layout/range.html': layoutBuilder
    };

    function buildRowV2(widget) {
        let newRow = {};
        widget.fields.forEach((field) => {
            if(rowBuilders.hasOwnProperty(field.template)){
                rowBuilders[field.template].call(null, field, newRow);
            }
        });
        newRow.surveyData = {
            NRPROGRAMACAOPESQ : widget.currentRow.NRPROGRAMACAOPESQ,
            NRQUESTIONARIO : widget.currentRow.NRQUESTIONARIO
        };
        return newRow;
    }
    
    
    function buildRow(widget) {
        let row = stripATchar(widget.currentRow);
        for (const key in row) {
            let field = widget.getField(key);
            if (field && field.template.includes("textarea")) {
                row[key] = {
                    value: row[key],
                    type: "open"
                };
            }
        }
        return row;
    }

    this.filterForAnon = function (widget) {
        if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true && getLocalVar('lastQuestAsw')) {
            let list = getLocalVar('lastQuestAsw');
            let toBeRemoved = [];
            list.forEach((it,key)=>{
                if(moment().format('DD/MM/YYYY') === it.date){
                    let toRemove = widget.dataSource.data.find(d=>d.NRPROGRAMACAOPESQ === it.NRPROGRAMACAOPESQ && d.NRQUESTIONARIO === it.NRQUESTIONARIO);
                    if(toRemove){
                        widget.dataSource.data.splice(widget.dataSource.data.indexOf(toRemove),1);
                    }    
                }else{
                    toBeRemoved.push(key);
                }
            });
            toBeRemoved.forEach(tb =>{
                list.splice(tb,1);
            });
            setLocalVar('lastQuestAsw',list);
        }
    };
    this.pesquisaSubmmit = function (itemPesquisaWidget) {
        if (itemPesquisaWidget.isValid()) {
            if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true) {
                let last = {
                    "date": moment().format('DD/MM/YYYY'),
                    "NRQUESTIONARIO": itemPesquisaWidget.currentRow.NRQUESTIONARIO,
                    "NRPROGRAMACAOPESQ": itemPesquisaWidget.currentRow.NRPROGRAMACAOPESQ
                };
                if(getLocalVar('lastQuestAsw')){
                    let arr = getLocalVar('lastQuestAsw');
                    arr.push(last);
                    setLocalVar('lastQuestAsw', arr);
                }else{
                    setLocalVar('lastQuestAsw', [last]);
                }
            }

            General.callRestEngine('/pesquisaSubmmit', 'Row', buildRowV2(itemPesquisaWidget)).then(
                function () {
                    MessageService.showMessageByCode('MSG_SENT_RATING_WITH_SUCCESS', null, {type:'success'});
                    ScreenService.openWindow(pesquisaContainerName);
                    pesquisasWidget.reload();
                    currentPesq = {};
                }
            );
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('PesquisaController', PesquisaController);
});