function CardapioController(ApplicationContext, ScreenService, MessageService) {
    let sf = this;
    const {LocalPlateDataService, templateManager} = ApplicationContext;

    const getLimitDate = (limitHours, limitDays) => {
        const [hours, minutes] = limitHours.split(':');
        const limitTime = moment();
        const days = parseInt(limitDays);
        limitTime.add({minutes, hours, days});
        return limitTime;    
    };

    this.getConfigSize = () => {
        if (getLocalVar('IDTAMCARDIMG')) {
            if (getLocalVar('IDTAMCARDIMG') === 'S') {
                return 'md';
            } else if (getLocalVar('IDTAMCARDIMG') === 'H') {
                return 'sm';
            }
        }
    };

    this.getTitleFiledComposition = (servdata, field) => {
        return servdata[field.code] ? servdata[field.code] + ' | ' + servdata[field.name] : servdata[field.name];
    };

    this.getIDEXIBENUTR = () => {
        return (getLocalVar('IDEXIBENUTR') && getLocalVar('IDEXIBENUTR') === 'N') ? false : true;
    };

    this.getTimeFromDataSourceFilter = (widget) => {
        widget.dataSourceFilter.find(f => f.name === widget.dateName);

        let curDate = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO');
        curDate = moment(curDate.value, 'DD/MM/YYYY').format('YYYY-MM-DD') === "Invalid date" ? moment().format('YYYY-MM-DD') : moment(curDate.value, 'DD/MM/YYYY').format('YYYY-MM-DD');
        widget.dateFiltred = curDate;

        return moment(curDate, 'DD/MM/YYYY').format('DD/MM/YYYY') === "Invalid date" ? ' ' + moment(curDate, 'YYYY-MM-DD').format('ddd - DD/MM') : ' ' + moment(curDate.value, 'DD/MM/YYYY').format('ddd - DD/MM');
    };

    this.changeTimeFromDataSourceFilter = (widget) => {
        if (widget.dateFiltred > widget.DTMAX) {
            widget.dateFiltred = widget.DTMAX;
        } else if (widget.dateFiltred < widget.DTMIN) {
            widget.dateFiltred = widget.DTMIN;
        }
        widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value = moment(widget.dateFiltred, 'YYYY-MM-DD').format('DD/MM/YYYY');
        sf.getTimeFromDataSourceFilter(widget);
        widget.reload();
    };

    this.moveOneDay = (direction, widget) => {
        let actualDate = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        actualDate = actualDate === "@moment().format('DD/MM/YYYY')@" ? moment().format('DD/MM/YYYY') : actualDate;

        let newDate;
        if (direction) {
            //foward
            newDate = moment(actualDate, 'DD/MM/YYYY').add(1, 'd');
        } else {
            newDate = moment(actualDate, 'DD/MM/YYYY').subtract(1, 'd');
        }
        widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value = newDate.format('DD/MM/YYYY');
        sf.getTimeFromDataSourceFilter(widget);
        widget.reload();
    };

    this.showTime = (time) => {
        return time === ': - :' ? '' : time;
    };

    this.showLimitLabel = (serv) => {
        let status = (serv.HRINICIO === ':') ? ScreenService.i18n('Adiantamento de: ') : ScreenService.i18n('Antecedência de: ');
        return serv.HRLIMSEGPEDIDO !== null ? status + serv.HRLIMSEGPEDIDO.substring(0, 2) + ':' + serv.HRLIMSEGPEDIDO.substring(2, 4) : '';
    };

    this.hasImage = (objToFind, listConfig) => {
        return (listConfig.imageURL && (objToFind.IMGTHUMB || objToFind[listConfig.imageURL])) || objToFind[listConfig.bs4IMG];
    };

    this.getBackImg = (objToFind, listConfig) => {
        let url = getLocalVar("qrCodeURL");
        let defaultImgUrl = "assets/images/icons/no-picture-camera-100.png";
        if (listConfig.imageURL && !listConfig.bs4IMG) {
            // let thumbImg = objToFind.IMGTHUMB;
            let thumbImg = objToFind.IMGSERVICO;
            let imgPath = thumbImg ? thumbImg : objToFind[listConfig.imageURL];
            return imgPath ? imgPath : defaultImgUrl;
        } else {
            let imgPath = objToFind[listConfig.bs4IMG];
            return imgPath ? 'data:image/png;base64,' + imgPath : defaultImgUrl;
        }
    };
    
    
    this.getPlateImage = (plate, listConfig) => {
        let baseUrl =  window.__backendURL || (window.__backendURL = getLocalVar("backendURL"));
        const {NRFOTOPRATOBLOB,URL_FOTOPRATO} = plate;
        const NRORG = getLocalVar('userData').NRORG;
        const url = `${baseUrl}/plateimage?NRFOTOPRATOBLOB=${NRFOTOPRATOBLOB}&NRORG=${NRORG}`;
        let urlMobile = localStorage.getItem('fileServerURL');
        let urlImagem;
        
        if(URL_FOTOPRATO == null){
            urlImagem = url;
        }else{
            urlImagem = urlMobile+URL_FOTOPRATO;
        }
        
        return NRFOTOPRATOBLOB ? {
            "background-image": `url(${urlImagem})`
        } : {};
    };
    
    this.goToDetails = (servdata, widget) => {
        let widgets = widget.container.widgets;
        let acwidget = widgets.indexOf(widget);

        if (widgets[acwidget + 1]) {
            let widgetToGo = widgets[acwidget + 1];

            let IDEXIBENUTR = getLocalVar('IDEXIBENUTR') ? getLocalVar('IDEXIBENUTR') : 'S';
            if (widgetToGo.name === "nutrients" && IDEXIBENUTR === 'N') {
                return false; //Should not show nutrients screen when not available (IDEXIBENUTR === 'N')
            }

            widget.isVisible = false;
            widgetToGo.container.label = servdata[widgetToGo.label];
            let filterToInherit = widgetToGo.filterToInherit;
            widgetToGo.dataSourceFilter = [];

            if (filterToInherit && widgetToGo.name !== "nutrients") {
                filterToInherit.forEach(fil => {
                    if (fil === 'DTCARDAPIO') {
                        servdata[fil] = widgets[acwidget].dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
                    }
                    widgetToGo.dataSourceFilter.push({
                        'name': fil,
                        'operator': '=',
                        'value': servdata[fil] || null
                    });
                });
            } else if (widgetToGo.name === "nutrients") {
                widgetToGo.filterToInherit.forEach(fil => {
                    widgetToGo.dataSourceFilter.push({
                        'name': fil,
                        'operator': '=',
                        'value': servdata[fil] || widget.currentRow.servdata[fil]
                    });
                });
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRORG',
                    'operator': '=',
                    'value': getLocalVar('userData').NRORG
                });
            }
            if (widgetToGo.name === "plates") {
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRUSUARIOAPP',
                    'operator': '=',
                    'value': getLocalVar('userData').NRUSUARIOAPP
                });
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRLOCALPESQ',
                    'operator': '=',
                    'value': getLocalVar('NRLOCALPESQ')
                });
            }

            widgetToGo.isVisible = true;
            widgetToGo.currentRow.servdata = servdata;
            widgetToGo.activate();
            widgetToGo.reload();
        }
    };

    this.rateIt = (widget, star) => {
        let stars = widget.stars;
        stars.forEach(st => {
            if (st === star) {
                star.active = true;
            } else {
                st.active = false;
            }
        });
        widget.plateToRate.newAv = star.value;
    };

    this.doAction = (code, widget) => {
        if (widget.plateToRate && widget.plateToRate.newAv) {

            widget.stars.forEach(st => {
                st.active = false;
            });
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            $scope.$eval(code, Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            }));
        } else {
            MessageService.showMessageByCode('MSG_INCOMPLETE_RATING');
        }

    };

    this.cancelRate = async (widget) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            widget.plateToRate = null;
            widget.stars.forEach(st => {
                st.active = false;
            });
            templateManager.updateTemplate();
        }
    };

    this.setStarValue = (widget) => {
        let stars = widget.stars;
        stars.forEach(st => {
            st.active = false;
        });
        let vl = widget.currentRow.servdata.VRAVALIACAO;
        let svl = stars.find(st => st.value == Math.round(vl));
        if (svl) {
            svl.active = true;
        }
    };

    function addServDataToPlate(plate, widget) {
        plate.CDFILCENTPME = widget.currentRow.servdata.CDFILCENTPME;
        plate.CDFILIAL = widget.currentRow.servdata.CDFILIAL;
        plate.CDSERVFILI = widget.currentRow.servdata.CDSERVFILI;
        plate.CDDIETPAD = widget.currentRow.servdata.CDDIETPAD;
        plate.CDTIPOCARD = widget.currentRow.servdata.CDTIPOCARD;
        plate.DTCARDAPIO = widget.currentRow.servdata.DTCARDAPIO;
        plate.NRORG = widget.currentRow.servdata.NRORG;
    }

    function getIDConsumoFromStorage(plate) {
        const localPlate = LocalPlateDataService.findPlate(plate) || {};
        return localPlate.IDCONSUMO || 'N';
    }

    this.isConsmBookmarkable = widget => {
        let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');
        let notToday = dtfil.format('YYYYMMDD') >= moment().format('YYYYMMDD');

        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0 && notToday) {
            if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true && getLocalVar('savedSrvCons')) {
                if (getLocalVar('savedSrvCons').length) {
                    let list = getLocalVar('savedSrvCons');
                    list.forEach(service => {
                        // jshint ignore:start                            
                        widget.dataSource.data.forEach(serv => {
                            serv.DTCARDAPIO = dtfil.format('DD/MM/YYYY');
                        });
                        let data = widget.dataSource.data.find(sc => sc.CDFILIAL == service.CDFILIAL && sc.CDSERVFILI == service.CDSERVFILI && sc.CDTIPOCARD == service.CDTIPOCARD && sc.DTCARDAPIO == service.DTCARDAPIO && sc.NRORG == service.NRORG);
                        if (data) {
                            data.IDCONSUMO = service.IDCONSUMO;
                            data.NMUSUARIO = service.NMUSUARIO;
                            data.HRCONSUMO = service.HRCONSUMO;
                        }
                        // jshint ignore:end
                    });
                }
            }
        }
        // }
    };

    this.checkIfPlateCONCFMES = (plate) => {
        let savedIDCONCNFMES = getLocalVar('IDCONCNFMESLIST');
        let retorno = false;
        if (savedIDCONCNFMES) {
            savedIDCONCNFMES.forEach(sicm => {
                if (plate.IDCONCNFMES == 'S' && plate.NRCARDAPIO == sicm.NRCARDAPIO && plate.CDTIPOPRAT == sicm.CDTIPOPRAT && plate.CDPRATO != sicm.CDPRATO) {
                    retorno = true;
                }
            });
        }
        return retorno;
    };

    this.isRatable = (widget) => {
        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0) {
            for (let i = 0; i < widget.dataSource.data.length; i++) {
                let plate = widget.dataSource.data[i];
                addServDataToPlate(plate, widget);
                let rateReturn = ['', false];
                let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
                dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');

                plate.DTCARDAPIOFormated = dtfil.format('DD/MM/YYYY');
                if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true) {
                    const plateForLocalStorageLookUp = Object.assign({}, plate, {DTCARDAPIO: dtfil.format('DD/MM/YYYY')});
                    let isRatable = !LocalPlateDataService.isPlateRated(plateForLocalStorageLookUp);

                    if (isRatable) {
                        if (dtfil.isSameOrBefore(moment(), 'days')) {
                            rateReturn = ['R'];
                        } else {
                            rateReturn = ['NT', 'MSG_CANT_RANK_PLATE_TODAY'];
                        }
                    } else {
                        rateReturn = ['AR', 'MSG_PLATE_ALREADY_RANKED'];
                    }

                    plate.IDCONSUMO = getIDConsumoFromStorage(plateForLocalStorageLookUp);
                } else {
                    if (plate.RESPONDIDO === 'N') {
                        if (dtfil.isSameOrBefore(moment(), 'days')) {
                            rateReturn = ['R'];
                        } else {
                            rateReturn = ['NT', 'MSG_CANT_RANK_PLATE_TODAY'];
                        }
                    } else {
                        rateReturn = ['AR', 'MSG_PLATE_ALREADY_RANKED'];
                    }
                }

                plate.rateReturn = rateReturn;
                plate.bookMark = dtfil.isSameOrAfter(moment(), 'days');
                plate.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;

                widget.dataSource.data[i] = plate;
                
            }
        }
    };

    this.doTheRate = (plate, widget, e) => {
        if (plate.rateReturn && plate.rateReturn[0] === 'AR' || plate.rateReturn[0] === 'NT') {
            MessageService.showMessageByCode(plate.rateReturn[1]);
        } else {
            widget.plateToRate = plate;
        }
        e.stopPropagation();
    };

    this.openBookMark = async (data, widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        let blockReservation = this.verifyBlockReservationAnon();
        
        if(blockReservation) {
            MessageService.showMessageByCode("MSG_BLOCK_RESERVATION");
            e.stopPropagation();
            return;
        } 
        if(validaQrCode == true){
            const currentPlate = data;
            const plates = widget.dataSource.data;
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            
            if(!data.DTCARDAPIO){
                let dtCard = widget.dataSourceFilter.find(f=>f.name=="DTCARDAPIO").value;
    
                if(dtCard[0]=="@" && dtCard[dtCard.length-1]=="@"){
                    data.DTCARDAPIO = $scope.$eval(dtCard.slice(1,-1), Object.assign({}, ApplicationContext, {
                        args: {
                            owner: $scope.widget
                        },
                        moment : moment
                    }));
                } else {
                    data.DTCARDAPIO = dtCard;
                }    
            } else {
                data.DTCARDAPIO = data.DTCARDAPIO ? data.DTCARDAPIO : moment().format("DD/MM/YYYY");
            } 
            
    
            e.stopPropagation();
            data.HRCONSUMO = null;
            widget.dataBookmarking = data;
            widget.dataBookmarking.NMUSUARIO = getLocalVar('userData').isAnon == false ? getLocalVar('userData').NAME : null;
            widget.dataBookmarking.nameIsFixed = getLocalVar('userData').isAnon == false;
            
            const {previousBookmarkDetails} = widget;
            let {CDSERVFILI, DTCARDAPIO} = data;
            
    
            DTCARDAPIO = DTCARDAPIO.includes('moment') ? moment().format('DD/MM/YYYY') : DTCARDAPIO;
            
            if(previousBookmarkDetails && previousBookmarkDetails.CDSERVFILI === CDSERVFILI && 
                previousBookmarkDetails.DTCARDAPIO === DTCARDAPIO){
                widget.dataBookmarking.HRCONSUMO = previousBookmarkDetails.HRCONSUMO;
                widget.dataBookmarking.NMUSUARIO = previousBookmarkDetails.NMUSUARIO;
            }
            templateManager.updateTemplate();
        }
    };

     this.converteDataCardapio = (data) =>{
         let ano = data.substring(6, 10);
         let mes = data.substring(3, 5);
         let dia = data.substring(0, 2);
         
         let dataConvertida = ano + "-" + mes + "-" + dia;

         return dataConvertida;
    };
    
    
    this.defineDataConsumo = (widget) =>{
        
        let dataCardapio;
        let hrInicio;
        let hrFinal;
        let hrIniPreparo;
        
        if(widget.name == 'mainWidget'){
            
            dataCardapio = moment(widget.dateFiltred).format("DD/MM/YYYY");
            hrInicio =  widget.dataBookmarking.HRINICIO;
            hrFinal = widget.dataBookmarking.HRFINAL;
            hrIniPreparo = widget.dataBookmarking.HRINICIOPREPARO;
        }else{
            dataCardapio = widget.dataBookmarking.DTCARDAPIOFormated;
            hrInicio =  widget.currentRow.servdata.HRINICIO;
            hrFinal = widget.currentRow.servdata.HRFINAL;
            hrIniPreparo = widget.currentRow.servdata.HRINICIOPREPARO;
            
        }
        
        let dataConsumo;
        
        let dtCardapioConvertida = sf.converteDataCardapio(dataCardapio);
        
        if(!hrInicio && hrFinal || hrIniPreparo > hrInicio){
            dataConsumo = moment(dtCardapioConvertida).add(1, "days");
            dataConsumo = moment(dataConsumo).format("YYYY-MM-DD");
        }else if(hrIniPreparo < hrInicio && hrInicio > hrFinal){
            let dtMin =  sf.minDataConsumo(widget);
            let dtMax = sf.maxDataConsumo(widget);
            widget.dataBookmarking.DTMINCONSUMO = dtMin;
            widget.dataBookmarking.DTMAXCONSUMO = dtMax;
            dataConsumo = dtCardapioConvertida;
            
        }else if(!hrInicio && !hrFinal){
            dataConsumo = dtCardapioConvertida;
        }else{
            dataConsumo = dtCardapioConvertida;
        }
        return dataConsumo;
    };
    
     this.minDataConsumo = (widget) =>{
        let dataConsumo = sf.verificaDataConsumo(widget);
        
        let dataCardapio;
        
        if(widget.name == 'mainWidget'){

            dataCardapio = sf.converteDataCardapio(moment(widget.dateFiltred).format("DD/MM/YYYY"));

        }else{
            dataCardapio = sf.converteDataCardapio(widget.dataBookmarking.DTCARDAPIOFormated);
        }
        
        if(dataConsumo == false){
            return dataCardapio;
        }else{
            return false;
        }
        
    };
    
    this.maxDataConsumo = (widget) =>{
        let dataConsumo = sf.verificaDataConsumo(widget);
        
        let dataCardapio;
        
        if(widget.name == 'mainWidget'){

            dataCardapio = sf.converteDataCardapio(moment(widget.dateFiltred).format("DD/MM/YYYY"));

        }else{
            dataCardapio = sf.converteDataCardapio(widget.dataBookmarking.DTCARDAPIOFormated);
        } 
        
        if(dataConsumo == false){
            dataCardapio = moment(dataCardapio).add(1, "days");
            dataCardapio = moment(dataCardapio).format("YYYY-MM-DD");
            return dataCardapio;
        }else{
            return false;
        }
    };
    
    
    this.verificaDataConsumo = (widget) =>{
    
        let hrInicio;
        let hrFinal;
        let hrIniPreparo;
        
        if(widget.name == 'mainWidget'){
            hrInicio =  widget.dataBookmarking.HRINICIO;
            hrFinal = widget.dataBookmarking.HRFINAL;
            hrIniPreparo = widget.dataBookmarking.HRINICIOPREPARO;
        }else{
            hrInicio =  widget.currentRow.servdata.HRINICIO;
            hrFinal = widget.currentRow.servdata.HRFINAL;
            hrIniPreparo = widget.currentRow.servdata.HRINICIOPREPARO;
        }
        
        
        let result;
        
        if(!hrInicio || hrIniPreparo > hrInicio){
            result = true;
        }else if(hrIniPreparo < hrInicio && hrInicio > hrFinal){
            result = false;
        }else{
            result = true;
        }
        
        return result;
        
    };
    
    
    
    this.confirmBookMark = async (data, widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            const serviceData = (widget.dataBookmarking.HORAS_LIMITE && widget.dataBookmarking) || (widget.currentRow.servdata || widget.dataBookmarking);
    
            let now = moment();
            let daysToReserve = Number(serviceData.NRLIMSEGPEDIDO);
            var insertPlate = false;
            
            let hoursToReserve = 0;
            if(serviceData.HRLIMSEGPEDIDO){
                hoursToReserve = serviceData.HRLIMSEGPEDIDO.split('');
                hoursToReserve.splice(2,0,':');
                hoursToReserve = moment.duration(hoursToReserve.join('')).asHours();
            }
    
            
            const formatedDTCARDAPIO = serviceData.DTCARDAPIO.includes('moment') ? moment().format('DD/MM/YYYY') : serviceData.DTCARDAPIO;
            let dtConsumo = moment(widget.dataBookmarking.DTCONSUMO).format("DD/MM/YYYY");
            const formatedDTCONSUMO = dtConsumo.includes('moment') ? moment().format('DD/MM/YYYY') : dtConsumo;
            
            let reservationTime = moment(formatedDTCONSUMO + ' ' + widget.dataBookmarking.HRCONSUMO, 'DD/MM/YYYY HH:mm');
            
            if (Array.isArray(data)) {
                data.forEach((plate) => {
                    plate.DSCONSUMO = widget.dataBookmarking.DSCONSUMO || null;
                    plate.HRCONSUMO = widget.dataBookmarking.HRCONSUMO ? widget.dataBookmarking.HRCONSUMO.replace(':', '') : null;
                    plate.DTCARDAPIO = formatedDTCARDAPIO;
                    plate.NMUSUARIO = widget.dataBookmarking.NMUSUARIO;
                });
                insertPlate = true;
            } else {
                data.DSCONSUMO = data.DSCONSUMO || null;
                data.HRCONSUMO = data.HRCONSUMO ? data.HRCONSUMO.replace(':', '') : null;
                widget.dataBookmarking.DTCARDAPIO = formatedDTCARDAPIO;
            } 
    
    
            let minTime = null;
            let endServiceTime = null;
            
            if ((serviceData.HRINICIO && serviceData.HRINICIO != ":") && 
                (serviceData.HRINICIOPREPARO < serviceData.HRINICIO)&& (serviceData.HRINICIO < serviceData.HRFINAL) || 
                (serviceData.HRINICIO == serviceData.HRINICIOPREPARO)) {
                       
                minTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRINICIO, 'DD/MM/YYYY HH:mm');
                endServiceTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRFINAL, 'DD/MM/YYYY HH:mm');
                
            }else if(serviceData.HRINICIOPREPARO > serviceData.HRINICIO){
                minTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRINICIO, 'DD/MM/YYYY HH:mm');
                minTime.add(1, 'days'); 
                
                endServiceTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRFINAL, 'DD/MM/YYYY HH:mm');
                endServiceTime.add(1, 'days');
                
            }else if((serviceData.HRINICIOPREPARO < serviceData.HRINICIO)&&(serviceData.HRINICIO > serviceData.HRFINAL)){
                minTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRINICIO, 'DD/MM/YYYY HH:mm');
                 
                endServiceTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRFINAL, 'DD/MM/YYYY HH:mm');
                endServiceTime.add(1, 'days');
            }

            if(minTime && endServiceTime.isBefore(minTime)){
                endServiceTime.add(1,'days');
                let endHourService = moment(endServiceTime.format('HH:mm:ss'), 'HH:mm:ss');
                let hourReservation = moment(reservationTime.format('HH:mm:ss'), 'HH:mm:ss');
                if(reservationTime.isBefore(endServiceTime) &&  moment('00:00:00', 'HH:mm:ss').isSameOrBefore(hourReservation) && hourReservation.isSameOrBefore(endHourService)){
                    reservationTime.add(1, 'days');
                }
            }

 
            if(widget.dataBookmarking.NMUSUARIO && widget.dataBookmarking.HRCONSUMO){
                if (minTime && reservationTime.isSameOrAfter(minTime)) {
                    minTime.seconds(0);
                    minTime.subtract(daysToReserve, 'days');
                    minTime.subtract(hoursToReserve, 'hours');
                    
                    let dtMaxConsumo;
                    
                    if(widget.dataBookmarking.DTMAXCONSUMO){
                        dtMaxConsumo = moment(widget.dataBookmarking.DTMAXCONSUMO).format("DD/MM/YYYY");
                    }else{
                        dtMaxConsumo = moment(widget.dataBookmarking.DTCONSUMO).format("DD/MM/YYYY");
                    }
                    
                    let formatedDTMAXCONSUMO = dtMaxConsumo.includes('moment') ? moment().format('DD/MM/YYYY') : dtMaxConsumo;
                    formatedDTMAXCONSUMO = moment(formatedDTMAXCONSUMO + ' ' + "23:59:59", 'DD/MM/YYYY HH:mm:ss');
                    
                    
                   if(reservationTime > formatedDTMAXCONSUMO){
                        
                        MessageService.showMessageByCode('MSG_SELECT_AFTER_DATE_MAX', {PARAM: formatedDTMAXCONSUMO.format('DD/MM/YYYY')}, {type:'error'});
                        widget.dataBookmarking.HRCONSUMO = null;
                        
                    }else if (now.isBefore(minTime)) {
                        if (!endServiceTime || (endServiceTime && reservationTime.isSameOrBefore(endServiceTime))) {
                            if (insertPlate) sf.operationDataPlate('MyMenuCardapioController.saveConsData()', widget, e);
                            else sf.bookMark(data, widget, e);
                            if(widget.name === 'plates'){
                                setLocalVar('HRCONSUMO', widget.dataBookmarking.HRCONSUMO);
                                setLocalVar('DSCONSUMO', widget.dataBookmarking.DSCONSUMO);
                                const userData = getLocalVar('userData');
                                widget.previousBookmarkDetails = {
                                    HRCONSUMO:widget.dataBookmarking.HRCONSUMO, 
                                    NMUSUARIO: userData.isAnon ? widget.dataBookmarking.NMUSUARIO : userData.NAME, 
                                    DTCARDAPIO:formatedDTCARDAPIO, 
                                    CDSERVFILI:widget.dataBookmarking.CDSERVFILI
                                };
                            }
                        } else {
                            MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_INVALID_TIME', {PARAM: endServiceTime.format('HH:mm')}, {type:'error'});
                            widget.dataBookmarking.HRCONSUMO = null;
                        }
                    } else {
                        MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_INVALID_DATA', {PARAM: minTime.format('DD/MM/YYYY HH:mm')}, {type:'error'});
                        widget.dataBookmarking.HRCONSUMO = null;
                    }
    
                } else if (minTime && reservationTime.isSameOrBefore(minTime)) {
                    
                    let dtConsumoConvertida =  moment(widget.dataBookmarking.DTMINCONSUMO).format("DD/MM/YYYY");
                    let dataHoraPermitido = moment(dtConsumoConvertida + ' ' + serviceData.HRINICIO, 'DD/MM/YYYY HH:mm');
                    
                    dataHoraPermitido.seconds(0);
                    dataHoraPermitido.subtract(daysToReserve, 'days');
                    dataHoraPermitido.subtract(hoursToReserve, 'hours');
                    
                    let dtMinConsumo;
                    
                    if(widget.dataBookmarking.DTMINCONSUMO){
                        dtMinConsumo = moment(widget.dataBookmarking.DTMINCONSUMO).format("DD/MM/YYYY");
                    }else{
                        dtMinConsumo = moment(widget.dataBookmarking.DTCONSUMO).format("DD/MM/YYYY");
                    }
                    
                    
                    let formatedDTMINCONSUMO = dtMinConsumo.includes('moment') ? moment().format('DD/MM/YYYY') : dtMinConsumo;
                    formatedDTMINCONSUMO = moment(formatedDTMINCONSUMO + ' ' + "00:00:00", 'DD/MM/YYYY HH:mm:ss');
                    
                    if(reservationTime < formatedDTMINCONSUMO){
                
                        MessageService.showMessageByCode('MSG_SELECT_BEFORE_DATE_MIN', {PARAM: formatedDTMINCONSUMO.format('DD/MM/YYYY')}, {type:'error'});
                        widget.dataBookmarking.HRCONSUMO = null;
                    }else if(now.isBefore(dataHoraPermitido)){
 
                        if(reservationTime.isSameOrBefore(minTime)){
                            MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_SERVICE_WILL_START', {PARAM: minTime.format('HH:mm')}, {type:'error'});
                            widget.dataBookmarking.HRCONSUMO = null;
                        }else{
                            
                             if (!endServiceTime || (endServiceTime && reservationTime.isSameOrBefore(endServiceTime))) {
                                now.seconds(0);
                                now.add(daysToReserve, 'days');
                                now.add(hoursToReserve, 'hours');
                                if (reservationTime.isSameOrAfter(now)) {
                                    if (insertPlate) sf.operationDataPlate('MyMenuCardapioController.saveConsData()', widget, e);
                                    else sf.bookMark(data, widget, e);
                                    if(widget.name === 'plates'){
                                        setLocalVar('HRCONSUMO', widget.dataBookmarking.HRCONSUMO);
                                        setLocalVar('DSCONSUMO', widget.dataBookmarking.DSCONSUMO);
                                        const userData = getLocalVar('userData');
                                        widget.previousBookmarkDetails = {
                                            HRCONSUMO:widget.dataBookmarking.HRCONSUMO, 
                                            NMUSUARIO: userData.isAnon ? widget.dataBookmarking.NMUSUARIO : userData.NAME, 
                                            DTCARDAPIO:formatedDTCARDAPIO, 
                                            CDSERVFILI:widget.dataBookmarking.CDSERVFILI
                                        };
                                    }
                                } else {
                                    MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_TIME_BIGGER_THAN', {PARAM: now.format('DD/MM/YYYY HH:mm')}, {type:'error'});
                                    widget.dataBookmarking.HRCONSUMO = null;
                                } 
                            
                            } else {
                                MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_INVALID_TIME', {PARAM: endServiceTime.format('HH:mm')}, {type:'error'});
                                widget.dataBookmarking.HRCONSUMO = null;
                            }
                        }
                    }else{
                        
                        MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_SERVICE_WILL_START', {PARAM: minTime.format('HH:mm')}, {type:'error'});
                        widget.dataBookmarking.HRCONSUMO = null;
                    }
    
                } else {
                    MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_SERVICE_WILL_START', {PARAM: minTime.format('HH:mm')}, {type:'error'});
                    widget.dataBookmarking.HRCONSUMO = null;
                }
            } else {
                MessageService.showMessageByCode('MSG_FILL_THE_FIELD', null, {type:'error'});
                widget.dataBookmarking.HRCONSUMO = null;
            }
            templateManager.updateTemplate();
        }
        
    };

    this.bookMark = async (data, widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            e.stopPropagation();
            data.IDCONSUMO = data.IDCONSUMO === 'S' ? 'N' : 'S';
            if (data.IDCONSUMO === 'S') {
                MessageService.successNotificationByCode('MSG_CONFIRMED_RESERVATION');
            }
            
            let methodToEval = 'MyMenuCardapioController.saveSrvConsData()';
            
            if(data.IDCONSUMO === 'N') {
                let clickedTime = moment();
                let limitTime = this.getLimitTime(data, widget);
                if (clickedTime.isBefore(limitTime)) {
                    widget.dataBookmarking = angular.copy(data);
                    widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
                    let $scope = angular.element(document.getElementsByTagName("body")).scope();
                    $scope.$eval(methodToEval, Object.assign({}, ApplicationContext, {
                        args: {
                            owner: $scope.widget
                        }
                    }));
                } else {
                    MessageService.showMessageByCode("MSG_OVER_HOUR");
                    data.IDCONSUMO = 'S';
                }
            } else {
                widget.dataBookmarking = angular.copy(data);
                widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
                let $scope = angular.element(document.getElementsByTagName("body")).scope();
                $scope.$eval(methodToEval, Object.assign({}, ApplicationContext, {
                    args: {
                        owner: $scope.widget
                    }
                }));
            }
            templateManager.updateTemplate();
        }
    };

    this.operationDataPlate = (data, widget, e) => {
        e.stopPropagation();
        let $scope = angular.element(document.getElementsByTagName("body")).scope();
        $scope.$eval(data, Object.assign({}, ApplicationContext, {
            args: {
                owner: $scope.widget
            }
        }));
    };

    this.bookMarkPlate = (data, widget, e) => {
        
        let blockReservation = this.verifyBlockReservationAnon();
            if(blockReservation) {
                MessageService.showMessageByCode("MSG_BLOCK_RESERVATION");
                e.stopPropagation();
                return;
        } 
        
        e.stopPropagation();

        widget.dataBookmarking = angular.copy(data);
        widget.dataBookmarking.IDCONSUMO = 'S';
        widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
        widget.dataBookmarking.NMUSUARIO = getLocalVar('userData').isAnon == false ? getLocalVar('userData').NAME : null;
        widget.dataBookmarking.nameIsFixed = getLocalVar('userData').isAnon == false;
        if (widget.bookMarking == undefined) widget.bookMarking = [];
        
        if (data.IDCONCNFMES == 'S') {
            //Remove o prato marcado anteriormente do mesmo tipo de prato e modo opção ou não
            widget.bookMarking =  widget.bookMarking.filter(bm => {
                return ( bm.CDTIPOPRAT != widget.dataBookmarking.CDTIPOPRAT) || (bm.IDCONCNFMES != 'S');
            });
            let pratosMarcadosMesmoTipo =  widget.dataSource.data.filter(prato => {
                return prato.CDTIPOPRAT == widget.dataBookmarking.CDTIPOPRAT && prato.IDCONCNFMES == 'S' && prato.IDCONSUMO == 'S';
            });
            pratosMarcadosMesmoTipo.map(prato => prato.IDCONSUMO = 'N');
        }
        
        //Indica o prato atual para consumo
        data.IDCONSUMO = 'S';
        
        widget.bookMarking.push(widget.dataBookmarking);
    };

    this.getLimitTime = (data, widget) => {
        let mom = null;
        let objectWidget = null;
        let dtReserva = '';
            
        if(widget.name === 'plates') {
            objectWidget = widget.currentRow.servdata;
            const isAnon = getLocalVar('userData').isAnon ? getLocalVar('savedCons').filter((plate) => plate.CDPRATO == data.CDPRATO && plate.DTCARDAPIO == data.DTCARDAPIOFormated) : [];
            if (isAnon.length > 0) {
                dtReserva = isAnon[0].DTCARDAPIO + ' ' + isAnon[0].HRCONSUMO.substring(0, 2) + ':' + isAnon[0].HRCONSUMO.substring(2, 4);
            } else {
                dtReserva = data.DTRESERVA;
            }
        } else {
            objectWidget = data;
            const isAnon = getLocalVar('userData').isAnon ? getLocalVar('savedSrvCons').filter((service) => service.CDSERVFILI == data.CDSERVFILI && service.DTCARDAPIO == data.DTCARDAPIO) : [];
            if (isAnon.length > 0) {
                dtReserva = isAnon[0].DTCARDAPIO + ' ' + isAnon[0].HRCONSUMO.substring(0, 2) + ':' + isAnon[0].HRCONSUMO.substring(2, 4);
            } else {
                dtReserva = data.DTRESERVA;
            }
        }

        let timeObject = {
            'hours': objectWidget.HRLIMSEGPEDIDO == null ? '00' : objectWidget.HRLIMSEGPEDIDO.substring(0, 2),
            'minutes': objectWidget.HRLIMSEGPEDIDO == null ? '00' : objectWidget.HRLIMSEGPEDIDO.substring(2, 4)
        };

        if (objectWidget.HRINICIO === ':') {
            mom = moment(data.DTRESERVA, 'DD-MM-YYYY HH:mm');
        } else {
            const hrInicio = dtReserva.substring(0, 10) + ' ' + objectWidget.HRINICIO;
            mom = moment(hrInicio, 'DD-MM-YYYY HH:mm');
        }
        
        mom.subtract(timeObject);
        mom.subtract(objectWidget.NRLIMSEGPEDIDO, 'days');
        
        
        if(data.HRINICIOPREPARO && data.HRINICIO){
            if(data.HRINICIOPREPARO > data.HRINICIO){
                 mom = moment(mom).add(1, "days");
            }
            
        }else{
            
            if(widget.currentRow.servdata.HRINICIOPREPARO > widget.currentRow.servdata.HRINICIO ){
                mom = moment(mom).add(1, "days");
            }
        }
        
        
        return mom;
    };

    this.cancelBookMarkPlate = async (data, widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            e.stopPropagation();
            widget.dataBookmarking = angular.copy(data);
            widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
    
            if (widget.bookMarking && widget.bookMarking.length > 0) {
                const dataExistInArray = widget.bookMarking.filter((value) => value.NRCARDAPIO == widget.dataBookmarking.NRCARDAPIO &&
                                                                              value.CDFILIAL == widget.dataBookmarking.CDFILIAL &&
                                                                              value.NRSEQPRATCAR == widget.dataBookmarking.NRSEQPRATCAR);
                if(dataExistInArray.length > 0) {
                    data.IDCONSUMO = 'N';
                    widget.bookMarking.splice(widget.bookMarking.indexOf(dataExistInArray[0]), 1);
                }
            }
            templateManager.updateTemplate();
        }
    };

    this.cancelReservation = async (widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            const cancelPlate = widget.dataSource.data.filter((data) => data.IDCONSUMO == 'S');
            let clickedTime = moment();
            let limitTime = this.getLimitTime(cancelPlate[0], widget);
            const getNewInstanceMoment = (objMoment, format = 'DD-MM-YYYY HH:mm') => {
                let newInstance = moment(objMoment.format(format), format);
                newInstance.seconds(0);
                return newInstance;
            };
            if(clickedTime.isBefore(limitTime)) {
                this.doCancelReservation(widget, e);
            }else if (getNewInstanceMoment(clickedTime).isSameOrBefore(getNewInstanceMoment(limitTime))) {
                this.doCancelReservation(widget, e);
            } else {
                MessageService.showMessageByCode("MSG_OVER_HOUR");
            }
            templateManager.updateTemplate();
        }
    };

    this.doCancelReservation = async (widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            MessageService.confirmMessageByCode('MSG_CANCEL_RESERVE', null, {
                type: 'question',
                success: () => {
                    this.operationDataPlate('MyMenuCardapioController.deleteConsData()', widget, e);
                }
            });
        }
    };

    this.verifyBlockReservationAnon = () => {
        let idBloqueioReservaAnonima = getLocalVar("IDBLOQRESERVAANONIMO");
        let isAnon = getLocalVar("userData").isAnon;
        
        if(idBloqueioReservaAnonima == "S" && isAnon == true) {
            return true;
        } else {
            return false;
        }        
    };

    this.doReservation = async (widget, e) => {
        let validaQrCode = await ApplicationContext.engine.validaDtExpiracao();
        if(validaQrCode == true){
            widget.dataBookmarking.NMUSUARIO = getLocalVar('userData').isAnon == false ? getLocalVar('userData').NAME : null;
            widget.dataBookmarking.nameIsFixed = getLocalVar('userData').isAnon == false;
            widget.doReservation = true;
            e.stopPropagation();
            templateManager.updateTemplate();
        }
    };

    this.hasReservation = (widget, e ) => {
        const hasReservation = widget.dataSource.data.filter((data) => data.IDCONSUMO === 'S');
        return hasReservation.length > 0;
    };
    
    this.isSelectable = (currentElement, data) => {
        
        if(getLocalVar('userData').isAnon){
            data = getLocalVar("savedCons") || [];
        }
        
        if(currentElement.IDOPCCNFMES === 'N' && currentElement.IDCONCNFMES === 'S'){
            const plateType = currentElement.CDTIPOPRAT;
            return !data.some(d => d.CDTIPOPRAT === plateType && d.IDCONSUMO === 'S' && 
                                    d.IDOPCCNFMES === 'N' && d.IDCONCNFMES === 'S');
        } else {
            return true;
        }
    };
    
    this.getDtCardapioMoment = (widget) => {
        let dtCard = widget.dataSourceFilter.find(f=>f.name=="DTCARDAPIO").value;

        if(dtCard[0]=="@" && dtCard[dtCard.length-1]=="@"){
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            dtCard = $scope.$eval(dtCard.slice(1,-1), Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                },
                moment : moment
            }));   
        }

        if(!dtCard.includes('moment')){
            dtCard = moment(dtCard, "DD/MM/YYYY");
        }

        return dtCard;
    };   
    
    this.isUserAnon = () => getLocalVar('userData').isAnon;

    this.isBookmarked = (plate) => {
        const cachedBookmarks = getLocalVar("savedCons") || [];

        return cachedBookmarks.some(bm=>{
            return bm.CDPRATO == plate.CDPRATO &&
                bm.CDSERVFILI == plate.CDSERVFILI &&
                bm.CDFILIAL == plate.CDFILIAL &&
                bm.DTCARDAPIO == plate.DTCARDAPIOFormated && 
                bm.IDCONSUMO === 'S';
        });
    };
    
    this.initialData = (dataBookmarking) => {
        dataBookmarking.DSCONSUMO = getLocalVar('DSCONSUMO') != undefined ? getLocalVar('DSCONSUMO') : '';
        dataBookmarking.HRCONSUMO = getLocalVar('HRCONSUMO') ? getLocalVar('HRCONSUMO').substring(0, 2) + ':' + getLocalVar('HRCONSUMO').substring(2, 4) : '';
    };
    
    this.formatNumber = (plate) => {
        if(plate.QTMEDICASE != null && plate.QTMEDICASE != undefined) {
            if(plate.QTMEDICASE.charAt(0) === '.') {
                return "-\xa0\xa0 0" + plate.QTMEDICASE;
            }
            return "-\xa0\xa0" + plate.QTMEDICASE;
        }
    };
    
    this.formatNumberNutrient = (plateFieldValue) => {
        if(plateFieldValue && plateFieldValue.charAt(0) === '.') {
            return " -\xa0\xa0 0" + plateFieldValue;                    
        } else if(plateFieldValue) {
            return "-\xa0\xa0" + plateFieldValue;
        } else {
            return '';
        }
    };
    
    this.formatWidthForGlutenLactose = (fieldType) => {
        if(fieldType == 'rating'){
            return {'width':'100%', 'padding-right':'0px'};
        }
    };
    
    this.getSufix = (plateFields) => {
        for(let i=0; i<plateFields.length; i++) {
            if(plateFields[i].name == 'CALORIAS_KCAL_ROUND') {
                return plateFields[i].sufix;
            } 
        }
    };
    
        this.getClassType = (plateFields, plateFieldName) => {
        for(let i=0; i<plateFields.length; i++) {
            if(plateFields[i].name == plateFieldName) {
                return plateFields[i].type;
            } 
        }
    };
}