function ruiDateController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    var nomeDosMeses = [
        "janeiro",
        "fevereiro",
        "março",
        "abril",
        "maio",
        "junho",
        "julho",
        "agosto",
        "setembro",
        "outubro",
        "novembro",
        "dezembro"
    ];

    var diasDaSemanaAbreviado = ["dom", "seg", "ter", "qua", "qui", "sex", "sab"];

    var container = $element.get()[0];
    var dateInput = container.querySelector('#date-input');
    var calendarMode = 0x1;
    var yearMode = 0x2;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            dateInput.classList.add('rui-has-value');
            var dateArray = fieldValue.split('/'); //[0] == day, [1] == month, [2] == year
            dateContainer.date.day = parseInt(dateArray[0], 10);
            dateContainer.selectedDay = parseInt(dateArray[0], 10);
            dateContainer.date.month = parseInt(dateArray[1], 10) - 1;
            dateContainer.selectedMonth = parseInt(dateArray[1], 10) - 1;
            dateContainer.date.year = parseInt(dateArray[2], 10);
            dateContainer.selectedYear = parseInt(dateArray[2], 10);
            dateContainer.dateWasSelected = true;
            dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
            dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
            var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
            dateContainer.monthDisplay.textContent = monthYearString;
            dateInput.children[1].textContent = fieldValue;
            generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        }
        var label = ScreenService.i18n($scope.field.label);
        if ($scope.field.validations && $scope.field.validations.required) {
            label += " *";
        }
        dateInput.children[0].textContent = label;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            dateInput.classList.add('disabled');
        }
    };


    var dateContainer = container.querySelector('#calendar-container');
    var overlay = container.querySelector('#overlay');
    var modal = container.querySelector('#modalWindow');
    dateContainer.okButton = container.querySelector('#ok');
    dateContainer.cancelButton = container.querySelector('#cancel');
    dateContainer.table = container.querySelector('#calendar-table');
    dateContainer.days = container.querySelector('#days-container');
    dateContainer.monthDisplay = container.querySelector('#month-display');
    dateContainer.previousMonthButton = container.querySelector('#previous-month-button');
    dateContainer.nextMonthButton = container.querySelector('#next-month-button');
    dateContainer.topYearDisplay = container.querySelector('#top-year-display');
    dateContainer.topFullDateDisplay = container.querySelector('#top-full-date-display');
    dateContainer.yearPicker = container.querySelector('#year-picker-container');
    dateContainer.yearPickerOutterWrapper = container.querySelector('#year-picker-outter-wrapper');
    dateContainer.currentSelectedYearSpanInYearPicker = undefined;

    dateContainer.dateWasSelected = false;

    dateContainer.mode = calendarMode;

    dateContainer.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (dateContainer.dateWasSelected) {
            dateInput.classList.add('rui-has-value');
        }
        dateInput.classList.remove('active');
        dateContainer.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        dateInput.classList.add('active');
        dateInput.classList.remove('rui-has-value');
        toggleCalendarMode();
        overlay.classList.add('active');
        modal.classList.add('active');
        generateCalendar(dateContainer.date.month, dateContainer.date.year);
    };

    $scope.field.confirm = function(e) {
        dateContainer.dateWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            ((dateContainer.selectedDay) < 10 ? "0" + (dateContainer.selectedDay) : "" + (dateContainer.selectedDay)) +
            "/" + ((dateContainer.selectedMonth + 1) < 10 ? "0" + (dateContainer.selectedMonth + 1) : "" + (dateContainer.selectedMonth + 1)) +
            "/" + dateContainer.selectedYear;
        dateInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        dateInput.classList.add('rui-has-value');
        dateInput.classList.remove('active');
        dateContainer.validate();
    };


    overlay.onclick = $scope.field.cancel;
    dateInput.onclick = $scope.field.open;
    dateContainer.okButton.onclick = $scope.field.confirm;
    dateContainer.cancelButton.onclick = $scope.field.cancel;


    var dayClickHandler = function(eventInfo) {
        var previousActiveDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        previousActiveDay.classList.remove('active');
        dateContainer.selectedDay = Number.parseInt(eventInfo.target.textContent, 10);
        var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        activeDay.classList.add('active');
        dateContainer.date.day = dateContainer.selectedDay;
        dateContainer.date.month = dateContainer.selectedMonth;
        dateContainer.date.year = dateContainer.selectedYear;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    };

    function generateCalendar( /*number */ month, /*number */ year) /*number: phantomOffset*/ {
        //debugger;
        //dateContainer.table;
        //dateContainer.days.opacity = 0;
        while (dateContainer.days.firstChild) {
            dateContainer.days.removeChild(dateContainer.days.firstChild);
        }

        var phantomDay = document.createElement('span');
        phantomDay.classList.add('phantom-day');
        phantomDay.textContent = '00';

        var firstDayOfWeek = new Date(year, month, 1).getDay();
        dateContainer.phantomDaysOffset = firstDayOfWeek - 1;

        for (var i = 0; i < firstDayOfWeek; i++) {
            dateContainer.days.appendChild(phantomDay.cloneNode());
        }

        var lastDayOfMonth = new Date(year, month + 1, 0).getDate();

        for (var j = 1; j <= lastDayOfMonth; j++) {
            var day = document.createElement('span');
            day.textContent = "" + j; //((i < 10) ? "0" + i : "" + i);
            day.onclick = dayClickHandler;
            dateContainer.days.appendChild(day);
        }

        if (dateContainer.date.month === month &&
            dateContainer.date.year === year) {
            var phantomDaysOffset = new Date(year, month, 1).getDay() - 1;
            var activeDay = dateContainer.days.children[phantomDaysOffset + dateContainer.date.day];
            activeDay.classList.add('active');
        }

    }

    /*monthsToAdd = 1 vai para o proximo mes, e -1 para o anterior*/
    /* retorna o objeto da nova data*/
    var addMonths = function( /*number*/ monthsToAdd) {
        var newDate = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        newDate.setMonth(dateContainer.selectedMonth + monthsToAdd);
        var newString = buildMonthWithYearString(newDate.getMonth(), newDate.getFullYear());
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.selectedYear = newDate.getFullYear();
        dateContainer.selectedMonth = newDate.getMonth();
        dateContainer.selectedDay = newDate.getDate();
        generateCalendar(newDate.getMonth(), newDate.getFullYear());
        return newDate;
    };

    var buildMonthWithYearString = function( /*number */ month, /*number */ year) {
        return nomeDosMeses[month] + " de " + year;
    };

    var buildTopDisplayString = function() /* string */ {
        //return nomeDosMeses[month] + " de " + year;
        var date = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        var nomeDiaDaSemana = diasDaSemanaAbreviado[date.getDay()];
        var nomeDoMesAbreviado = nomeDosMeses[dateContainer.selectedMonth].substr(0, 3);
        return nomeDiaDaSemana + ", " + dateContainer.selectedDay + " de " + nomeDoMesAbreviado;
    };

    var setDateUsingJSDateObject = function( /* Date */ data) {
        dateContainer.date.year = data.getFullYear();
        dateContainer.date.month = data.getMonth();
        dateContainer.date.day = data.getDate();
    };

    dateContainer.previousMonthButton.onclick = function() {
        var newDate = addMonths(-1);
    };

    dateContainer.nextMonthButton.onclick = function() {
        var newDate = addMonths(1);
    };

    var toggleYearMode = function() {
        dateContainer.classList.remove('active');
        dateContainer.yearPickerOutterWrapper.classList.add('active');
        dateContainer.topYearDisplay.classList.add('active');
        dateContainer.topFullDateDisplay.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker.scrollIntoView();
    };

    var toggleCalendarMode = function() {
        dateContainer.classList.add('active');
        dateContainer.yearPickerOutterWrapper.classList.remove('active');
        dateContainer.topYearDisplay.classList.remove('active');
        dateContainer.topFullDateDisplay.classList.add('active');
    };

    var pickYear = function( /* clickEvent */ e) {
        var newYear = e.target.textContent;
        dateContainer.selectedYear = newYear;
        var newString = buildMonthWithYearString(dateContainer.selectedMonth, newYear);
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker = e.target;
        e.target.classList.add('active');
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.topYearDisplay.textContent = newYear;
        dateContainer.date.year = newYear;
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        toggleCalendarMode();
    };

    /*initial setup*/

    var today = new Date();
    var hoje = today.getDate();
    dateContainer.selectedDay = hoje;
    dateContainer.selectedMonth = today.getMonth();
    dateContainer.selectedYear = today.getFullYear();
    dateContainer.date = {
        year: today.getFullYear(),
        day: hoje,
        month: today.getMonth()
    };
    dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
    dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    dateContainer.monthDisplay.textContent = buildMonthWithYearString(today.getMonth(), today.getFullYear());
    generateCalendar(today.getMonth(), today.getFullYear());
    var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
    activeDay.classList.add('active');

    dateContainer.topYearDisplay.onclick = toggleYearMode;
    dateContainer.topFullDateDisplay.onclick = toggleCalendarMode;

    for (var i = 1970; i < 2051; i++) {
        var n = document.createElement("span");
        n.textContent = i;
        dateContainer.yearPicker.appendChild(n);
        n.onclick = pickYear;
    }

    dateContainer.yearPicker.currentYearNode = dateContainer.yearPicker.children[today.getFullYear() - 1970];
    dateContainer.yearPicker.currentYearNode.classList.add('active');
    dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;

    /******* ORIENTATION ********/

    if (window.orientation !== undefined) {
        window.addEventListener("orientationchange", function() {
            detectOrientationAndRotate();
        });
    }

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
    };

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        //
        dateInput.classList.remove('rui-has-value');
        var today = new Date();
        dateContainer.date.day = today.getDate();
        dateContainer.selectedDay = dateContainer.date.day;
        dateContainer.date.month = today.getMonth();
        dateContainer.selectedMonth = dateContainer.date.month;
        dateContainer.date.year = today.getFullYear();
        dateContainer.selectedYear = dateContainer.date.year;
        dateContainer.dateWasSelected = false;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
        var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
        dateContainer.monthDisplay.textContent = monthYearString;
        dateInput.children[1].textContent = '';
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        $scope.fieldRow = '';
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.yearPicker.currentYearNode.classList.add('active');
        dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;
        dateContainer.validate();
    };
}
