function ZhInnerSocket(ZhLogger) {

	var connection;
	var isOpen;
	var client;
	var host;
	var templateManager;
	var runningRetry;
	var listeners = [];
	var self = this;

	var setup = function(newConnection) {

		newConnection.onopen = function() {
			isOpen = true;
			self.notify({
				event: 'onOpen',
				data: true
			});
			//newConnection.send('Connected!');
		};

		newConnection.onerror = function(error) {
			ZhLogger.error('WebSocket error: ', error);
			self.notify({
				event: 'onError',
				data: true
			});
		};

		newConnection.onmessage = function(message) {
			//ZhLogger.debug(message.data);
			message = JSON.parse(message.data);
			if (client[message.event]) {
				client[message.event](message.data);
				if (templateManager.project.updateTemplateAfterWebSocketMessage) {
					updateTemplateAfterNotify();
				}
			}
			var parse = {
				"event": "onMessage",
				"data": message.data
			};
			self.notify(parse);
			parse = {
				"event": message.event,
				"data": message.data
			};
			self.notify(parse);
		};

		newConnection.onclose = function() {
			isOpen = false;
			runningRetry = false;
			self.notify({
				"event": "connectionLost",
				"data": true
			});
			if (!runningRetry) {
				runningRetry = true;
				setTimeout(self.retryConnection, 500);
			}
		};
	};

	this.create = function(socketClient, _host, _templateManager) {
		connection = new WebSocket(_host);
		client = new socketClient();
		host = _host;
		templateManager = _templateManager;
		setup(connection);
		return this;
	};

	this.retryConnection = function() {
		self.notify({
			"event": "retryConnection",
			"data": true
		});
		connection = new WebSocket(host);
		setup(connection);
		runningRetry = false;
	};

	this.send = function(message) {
		connection.send(message);
	};

	this.isOpen = function() {
		return isOpen;
	};

	this.close = function() {
		connection.close();
		isOpen = false;
	};

	this.addListener = function(listener) {
		//if it isn't there already, add it
		if (listeners.indexOf(listener) === -1) {
			listeners.push(listener);
		}
	};

	this.removeListener = function(listener) {
		//find 1st matching method's position
		var index = listeners.indexOf(listener);
		//if it was found...
		if (index > -1) {
			//...remove it from the array
			listeners.splice(index, 1);
		}
	};

	this.notify = function(message) {
		listeners.forEach(function(listener) {
			if (listener[message.event]) {
				listener[message.event](message.data);
			}
		});
		if (templateManager.project.updateTemplateAfterWebSocketMessage) {
			updateTemplateAfterNotify();
		}
	};
	
	var updateTemplateAfterNotify = Util.buildThrottleMethod(function () {
		templateManager.updateTemplate();
	}, 150);
}

function ZhSocketFactory(ZhLogger, templateManager) {
	this.create = function(socketClient, host) {
		return new ZhInnerSocket(ZhLogger).create(socketClient, host, templateManager);
	};
}